#
# License: BSD
#    https://raw.githubusercontent.com/stonier/vci/devel/LICENSE
#
##############################################################################
# Documentation
##############################################################################

"""
Configure version control index settings.
"""

##############################################################################
# Imports
##############################################################################

import argparse
import os
import sys
import urlparse

from . import common
from . import console

##############################################################################
# Constants
##############################################################################

DEFAULT_INDEX_URL = 'https://raw.githubusercontent.com/stonier/vci/repos/kinetic.yaml'

##############################################################################
# Library
##############################################################################


def get_index_url():
    filename = os.path.join(common.home(), "settings")
    try:
        f = open(filename, 'r')
    except IOError:
        return set_index_url(DEFAULT_INDEX_URL)
    index_url = f.read()
    f.close()
    return index_url


def set_index_url(index_url):
    '''
      Set a uri for the vcs index to use for retrieval.
    '''
    # could actually check that it is a valid uri though.
    filename = os.path.join(common.home(), "settings")
    f = open(filename, 'w+')
    try:
        f.write(index_url.encode('utf-8'))
    finally:
        f.close()
    return index_url

##############################################################################
# Command Parser
##############################################################################


def examples_string():
    examples = console.bold + "Examples\n\n" + console.reset  \
        + "  1) Display the current index url\n\n" \
        + console.cyan + "      vci config\n\n" + console.reset \
        + "  2) Set the index to a local file\n\n" \
        + console.cyan + "      vci config " + console.yellow + "file:///home/stonier/kinetic.yaml\n\n" + console.reset \
        + "  3) Set the index to a public github file\n\n" \
        + console.cyan + "      vci config " + console.yellow + "{0}\n\n".format(DEFAULT_INDEX_URL) + console.reset \
        + "  4) Reset the index to the default\n\n" \
        + console.cyan + "      vci config " + console.yellow + "--set-default\n\n" + console.reset
    return examples


def parse_args(args):
    """
    Execute the command given the incoming args.
    """
    if args.set_default:
        set_index_url(DEFAULT_INDEX_URL)
    elif args.url:
        # very simple validation (note netloc is empty for file://)
        result = urlparse.urlparse(args.url)
        if not result.scheme:
            console.logerror("invalid url ['{0}']".format(args.url))
            sys.exit(1)
        set_index_url(args.url)
    index_url = get_index_url()
    if args.no_colour:
        print(index_url)
    else:
        print("\n" + console.cyan + "URL " + console.reset + ": " + console.yellow + index_url + console.reset + "\n")


def add_subparser(subparsers):
    """
    Add our own argparser to the parent.

    :param subparsers: the subparsers factory from the parent argparser.
    """
    subparser = subparsers.add_parser(
        "config",
        description="""Manage the list of version control index url's and their keys.""",
        help="manage the version control index list",  # this shows in the parent parser
        formatter_class=argparse.RawDescriptionHelpFormatter
    )
    subparser.epilog = examples_string()
    subparser.set_defaults(func=parse_args)
    common.add_nocolour_argument(subparser)
    add = subparser.add_argument
    add('-d', '--set-default', action='store_true', default=False, help='Reset to the default url ({0})'.format(DEFAULT_INDEX_URL))
    add('url', type=str, nargs='?', default="", help='set the url for the index to use for retrieval')


#
# License: BSD
#    https://raw.githubusercontent.com/stonier/vci/devel/LICENSE
#
##############################################################################
# Documentation
##############################################################################

"""
Arg parse construction.
"""
##############################################################################
# Imports
##############################################################################

import argparse

from .__version__ import __version__
from . import console
from . import find
from . import config
from . import index_contents

##############################################################################
# Methods
##############################################################################


def version_string():
    return console.cyan + "Version" + console.reset + " : " + console.yellow + __version__ + console.reset


def get_parser():
    # Create a top level parser
    parser = argparse.ArgumentParser(
        description="version control index handling",
        epilog="And his noodly appendage reached forth to tickle the blessed...\n",
        formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('-v', '--version', action='version', version=version_string())
    subparsers = parser.add_subparsers(title='commands',
                                       help='valid commands for vci interactions')
    find.add_subparser(subparsers)
    config.add_subparser(subparsers)
    index_contents.add_subparser(subparsers)
    return parser



#
# License: BSD
#    https://raw.githubusercontent.com/stonier/vci/devel/LICENSE
#
##############################################################################
# Documentation
##############################################################################

"""
Common helpers for the vci commands.
"""
##############################################################################
# Imports
##############################################################################

import os
import urlparse

##############################################################################
# Methods
##############################################################################


def home():
    """
    Get directory location of '.vci' where configuration is stored.

    @return: path to the home directory
    @rtype: str
    """
    home_dir = os.path.join(os.path.expanduser('~'), '.vci')
    if not os.path.exists(home_dir):
        os.makedirs(home_dir)
    return home_dir


def is_url(source_uri):
    """
    Uses heuristics to check whether uri is a url (as opposed to a file path).

    File paths always have empty scheme (e.g. http, file, https) AND
    netloc (e.g. github.com).

    :param str source_uri: string representing web uri or file path
    :returns: bool
    """
    if source_uri is None or source_uri == '':
        return False
    parsed_uri = urlparse.urlparse(source_uri)
    if (parsed_uri.scheme == '' and
        parsed_uri.netloc == '' and
        '@' not in parsed_uri.path.split('/')[0]
        ):
        return False
    return True


def add_index_argument(subparser):
    """
    Oft used argument for overriding the vci index.

    :param subparser: an argparse instance
    """
    add = subparser.add_argument
    add('-i', '--index', action='store', default=None, help="override the currently stored index for this call only")


def add_nocolour_argument(subparser):
    """
    Oft used argument for disabling colour.

    :param subparser: an argparse instance
    """
    add = subparser.add_argument
    add('--no-colour', action='store_true', help="disable colour output")







import sys
from setuptools import find_packages
from setuptools import setup
from vci import __version__

# Setup installation dependencies, removing some so they
# can build on the ppa
install_requires = [
    'setuptools',
    'PyYAML',
    'vcstool',
]
if sys.version_info[0] == 2 and sys.version_info[1] <= 6:
    install_requires.append('argparse')


setup(
    name='vci',
    version=__version__,
    packages=find_packages(exclude=['tests*', 'docs*']),
    # check into catkin_tools/ckx_tools for a smarter, but complicated method
    install_requires=install_requires,
    author='Daniel Stonier',
    author_email='d.stonier@gmail.com',
    maintainer='Daniel Stonier',
    maintainer_email='d.stonier@gmail.com',
    url='http://github.com/stonier/vci',
    keywords=['catkin'],
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Topic :: Software Development :: Version Control',
        'Topic :: Utilities'
    ],
    description="version control management from an index of listed workspaces",
    long_description="Version control index handling from a yaml file on the internet (e.g. github).",
    license='BSD',
    # test_suite='tests',
    entry_points={
        'console_scripts': [
            'vci = vci:main',
        ],
    },
)
